@@ -11,8 +11,16 @@ import ipywidgets
 from IPython.display import display,clear_output
 import webbrowser,qrcode,base64
 
+USE_SHORT_QR_CODE=True
+
+def getUID():
+    qstr = str(uuid.uuid4())
+    if USE_SHORT_QR_CODE:
+        return "-".join(qstr.split("-")[:-1])
+    return qstr
+
 def makeQRxls(fn, n=10):
-    qdict = {"UIDs" : [uuid.uuid4() for _ in  range(n)] }
+    qdict = {"UIDs" : [getUID() for _ in  range(n)] }
     df = pd.DataFrame.from_dict(qdict)
     df.to_excel(fn, index=False)
 
@@ -248,26 +256,29 @@ def process_sample_lists(xls_fns, process_all_tabs=False, b_lim=4):
 
     return slist,transfer_list,bdict
 
-def read_OT2_layout(plate_slots, holder_slots):
+def read_OT2_layout(plate_slots, holder_slots, msg=None):
     """ 
     the arguments should be a comma-separated list of slot positions on the Opentron deck
     e.g. "1,2"  
     """
-    ssh_key = str(pathlib.Path.home())+"/.ssh/ot2_ssh_key"
-    if not os.path.isfile(ssh_key):
-        raise Exception(f"{ssh_key} does not exist!")
+    if msg is None:
+        ssh_key = str(pathlib.Path.home())+"/.ssh/ot2_ssh_key"
+        if not os.path.isfile(ssh_key):
+            raise Exception(f"{ssh_key} does not exist!")
 
-    cmd = ["ssh", "-i", ssh_key, "-o", "port=9999",
-           "root@localhost", "/var/lib/jupyter/notebooks/check_deck_config.py", 
-           "-h", holder_slots, "-p", plate_slots]
+        cmd = ["ssh", "-i", ssh_key, "-o", "port=9999",
+               "root@localhost", "/var/lib/jupyter/notebooks/check_deck_config.py", 
+               "-h", holder_slots, "-p", plate_slots]
 
-    ret = subprocess.run(cmd, capture_output=True)
+        ret = subprocess.run(cmd, capture_output=True)
 
-    if ret.returncode:
-        print(ret)
-        raise Exception("error executing check_deck_config.py")
-
-    ldict = json.loads(ret.stdout.decode().split("*****")[-1])
+        if ret.returncode:
+            print(ret)
+            raise Exception("error executing check_deck_config.py")
+        
+        msg = ret.stdout.decode()
+            
+    ldict = json.loads(msg.split("*****")[-1])
     
     return ldict
 
@@ -299,14 +310,14 @@ def generate_measurement_spreadsheet(fn, slist, holders, bdict):
         df1.to_excel(writer, index=False, sheet_name=tname)
         df2.to_excel(writer, index=False, sheet_name="UIDs")
             
-
-def generate_docs(ot2_layout, xls_fns,    
+def generate_docs(ot2_layout, xls_fns, ldict=None,   
                   run_name="test",
                   b_lim=4,
                   plate_type = "corning_96_wellplate_360ul_flat",
                   holder_type = "lix_3x_holder_c",
                   tip_type = "opentrons_96_tiprack_300ul",
-                  flow_rate_aspirate = 50, flow_rate_dispense = 50
+                  flow_rate_aspirate = 50, flow_rate_dispense = 50, 
+                  bottom_clearance = 1
                  ):
     """ ot2_layout should be a dictionary:
             {"plates" : "1,2",
@@ -316,8 +327,10 @@ def generate_docs(ot2_layout, xls_fns,
     print("Processing sample list(s) ...")
     slist,transfer_list,bdict = process_sample_lists(xls_fns, b_lim=b_lim)
     
-    print("Reading bar/QR codes, this might take a while ...")
-    ldict = read_OT2_layout(ot2_layout["plates"], ot2_layout["holders"])
+    if ldict is None:
+        print("Reading bar/QR codes, this might take a while ...")
+        ldict = read_OT2_layout(ot2_layout["plates"], ot2_layout["holders"])
+    
     print(ldict)
     
     holders = {}
@@ -351,6 +364,7 @@ def generate_docs(ot2_layout, xls_fns,
         protocol.append(f"    lbw{slot} = ctx.load_labware('{tip_type}', '{slot}')\n")
         tips.append(f"lbw{slot}")
     protocol.append(f"    pipet = ctx.load_instrument('p300_single', 'left', tip_racks=[{','.join(tips)}])\n")
+    protocol.append(f"    pipet.well_bottom_clearance.aspirate = {bottom_clearance}\n")
     protocol.append(f"    pipet.flow_rate.aspirate = {flow_rate_aspirate}\n")
     protocol.append(f"    pipet.flow_rate.dispense = {flow_rate_dispense}\n")
 
@@ -681,7 +695,7 @@ def validateHolderSpreadsheet(fn, proposal_id, SAF_id):
     ws1 = wb.create_sheet("UIDs")
     ws1.append(["holderName", "UID"])
     for i in range(len(hlist)):
-        ws1.append([hlist[i], str(uuid.uuid4())])
+        ws1.append([hlist[i], getUID()])
     wb.save(fn)
     
     print("Done.")
